/* Copyright (C) 2009-2010, Alexander Graf <agraf@znc.in>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#pragma once

/* pin configuration */
#define PINSEL0		0x00000050	/* UART RXD0, TXD0*/
#define PINMODE0	0x000000a0
#define PINSEL1		0x00000000
#define PINMODE1	0x00000000
#define PINSEL2		0x00000000
#define PINMODE2	0x00000000
#define PINSEL3		0x00000000
#define PINMODE3	0x00000000
#define PINSEL4		0x50000000	/* RAM CS[23] */
#define PINMODE4	0xa0000000
#define PINSEL5		0x55015555	/* RAM CAS, RAS, CLKOUT[01], DYCS[0:3], CKEOUT0, DQMOUT[0:3] */
#define PINMODE5	0xaa02aaaa
#define PINSEL6		0x55555555	/* RAM D[0:15] */
#define PINMODE6	0xaaaaaaaa
#define PINSEL7		0x55555555	/* RAM D[16:31] */
#define PINMODE7	0xaaaaaaaa
#define PINSEL8		0x55555555	/* RAM A[0:15] */
#define PINMODE8	0xaaaaaaaa
#define PINSEL9		0x55555554	/* RAM A[17:23], OE, WE, BLS[0:3], CS[01] */
#define PINMODE9	0xaaaaaaa8

#define OSCIFREQ	12000000
#define PLLMUL		24
#define	CPUDIV		4
#define	USBDIV		6
#define CPU_CLOCK	(OSCIFREQ * PLLMUL / CPUDIV)	/* 72 MHz */

/* clocks for timing of MAM
 * LPC24xx User Manual suggests the following timings:
 * cpu clock	cycles
 * < 20 MHz	1
 * 20 - 40 MHz	2
 * 40 - 60 MHz	3
 * > 60 MHz	4
 * Thus the following formula is a good approximation */
#define MAMTIM	(CPU_CLOCK / 20000000 + 1)

/* this is internal RAM. */
#define RAM_START	0x40000000
#define RAM_SIZE	(64 << 10)

#define EXTRAM_START	0xa0000000

/* Samsung 512K x 32Bit x 4 Banks Synchronous DRAM */
#ifdef SDRAM_K4S643232E
#include "ram_k4s643232e.h"
#endif

/* Micron 128 Mb SDRAM */
#ifdef SDRAM_MT48LC4M32B2
#include "ram_mt48lc4m32b2.h"
#endif

/* base address of uart0 used as console */
#define UART0_BASE	0xe000c000
