/* Copyright (C) 2009-2010, Alexander Graf <agraf@znc.in>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#include "cpu.h"

.text
.arm

.global _start
_start:
	/* exception vector */
	ldr	pc, =bootstrap		/* 0x00  svc  reset */
	ldr	pc, =unexpected		/* 0x04  und  undefined instructions */
	ldr	pc, =unexpected		/* 0x08  svc  software interrupt */
	ldr	pc, =unexpected		/* 0x0c  abt  prefetch abort */
	ldr	pc, =unexpected		/* 0x10  abt  data abort */
	nop				/* 0x14 reserved for future expansion */
	ldr	pc, =unexpected		/* 0x18  irq  interrupt (VIC-handled) */
	ldr	pc, =unexpected		/* 0x1c  fiq  fast interrupt */

unexpected:
	b	unexpected

bootstrap:
	ldr	r0,	=0xe01fc000	/* base of some system control regs */

	/* enable main oscillator */
	ldr	r1,	=(1 << 5)	/* main oscillator enable */
	str	r1,	[r0, #0x1a0]
1:	ldr	r1,	[r0, #0x1a0]	/* wait for oscillator to be ready */
	ands	r1, 	r1,	#(1 << 6)
	beq	1b

	/* initalize pll */
	ldr	r2,	=0xaa		/* feed sequence */
	ldr	r3,	=0x55
	ldr	r1,	=0		/* disable PLL */
	str	r1,	[r0, #0x080]
	str	r2,	[r0, #0x08c]
	str	r3,	[r0, #0x08c]
	ldr	r1,	=1		/* use main osci as PLL source */
	str	r1,	[r0, #0x10c]
	ldr	r1,	=((PLLMUL-1) | 1 << 16)	/* M: PLLMUL, D: 1 */
	str	r1,	[r0, #0x084]
	str	r2,	[r0, #0x08c]
	str	r3,	[r0, #0x08c]
	ldr	r1,	=1		/* enable */
	str	r1,	[r0, #0x080]
	str	r2,	[r0, #0x08c]
	str	r3,	[r0, #0x08c]
	ldr	r1,	=(CPUDIV-1)	/* CCLKCFG */
	str	r1,	[r0, #0x104]
	ldr	r1,	=(USBDIV-1)	/* USBCLKCFG */
	str	r1,	[r0, #0x108]
1:	ldr	r1,	[r0, #0x088]	/* wait for pll to be locked */
	ands	r1,	r1,	#(1 << 26)
	beq	1b
	ldr	r1,	=3		/* enable & connect pll */
	str	r1,	[r0, #0x080]
	str	r2,	[r0, #0x08c]
	str	r3,	[r0, #0x08c]

	/* set CCLK/1 as peripherial clock */
	ldr	r1,	=0x55515555
	str	r1,	[r0, #0x1a8]
	ldr	r1,	=0x11555455
	str	r1,	[r0, #0x1ac]

	/* initalize Memory Accelerator Module */
	ldr	r1,	=MAMTIM
	str	r1,	[r0, #0x004]
	ldr	r1,	=2		/* fully enable */
	str	r1,	[r0, #0x000]

	/* enable high speed GPIO at ports 0 and 1 */
	ldr	r1,	=(1 | 1 << 5)
	str	r1,	[r0, #0x1a0]

	/* configure PIN functions and modes */
	ldr	r0,	=0xe002c000
#define PIN(x)				\
	ldr	r1,	=PINSEL##x;	\
	str	r1,	[r0, #(x*4)];	\
	ldr	r1,	=PINMODE##x;	\
	str	r1,	[r0, #(0x40+x*4)]
	PIN(0)
	PIN(1)
	PIN(2)
	PIN(3)
	PIN(4)
	PIN(5)
	PIN(6)
	PIN(7)
	PIN(8)
	PIN(9)
#undef PIN

	/* initalize external SDRAM */
	ldr	r0,	=0xffe08000	/* EMC base */
	ldr	r1,	=1
	str	r1,	[r0, #0x000]	/* EMC enable */
	ldr	r1,	=R_RS
	str	r1,	[r0, #0x028]	/* set read strategy */
	ldr	r1,	=R_RASCAS
	str	r1,	[r0, #0x104]	/* RAS & CAS timings */
	ldr	r1,	=R_tRP
	str	r1,	[r0, #0x030]	/* tRP timing */
	ldr	r1,	=R_tRAS
	str	r1,	[r0, #0x034]	/* tRAS timing */
	ldr	r1,	=R_tSREX
	str	r1,	[r0, #0x038]	/* tSREX timing */
	ldr	r1,	=R_tAPR
	str	r1,	[r0, #0x03c]	/* tAPR timing */
	ldr	r1,	=R_tDAL
	str	r1,	[r0, #0x040]	/* tDAL timing */
	ldr	r1,	=R_tWR
	str	r1,	[r0, #0x044]	/* tWR timing */
	ldr	r1,	=R_tRC
	str	r1,	[r0, #0x048]	/* tRC timing */
	ldr	r1,	=R_tRFC
	str	r1,	[r0, #0x04c]	/* tRFC timing */
	ldr	r1,	=R_tXSR
	str	r1,	[r0, #0x050]	/* tXSR timing */
	ldr	r1,	=R_tRRD
	str	r1,	[r0, #0x054]	/* tRRD timing */
	ldr	r1,	=R_tMRD
	str	r1,	[r0, #0x058]	/* tMRD timing */
	ldr	r1,	=R_ADRMAP
	str	r1,	[r0, #0x100]	/* address mappings */
	/* init sequence */
	ldr	r1,	=0
1:	add	r1,	r1,	#1
	cmp	r1,	#128
	bne	1b
	ldr	r1,	=0x183
	str	r1,	[r0, #0x020]
	ldr	r1,	=0
1:	add	r1,	r1,	#1
	cmp	r1,	#128
	bne	1b
	ldr	r1,	=0x101
	str	r1,	[r0, #0x020]
	ldr	r1,	=1
	str	r1,	[r0, #0x024]
	ldr	r1,	=0
1:	add	r1,	r1,	#1
	cmp	r1,	#128
	bne	1b
	ldr	r1,	=19
	str	r1,	[r0, #0x024]
	ldr	r1,	=0x081
	str	r1,	[r0, #0x020]
	ldr	r1,	=(0xa0000000 | (0x32 << 12))
	ldr	r2,	[r1]
	ldr	r1,	=0x000
	str	r1,	[r0, #0x020]
	ldr	r1,	[r0, #0x100]
	orr	r1,	r1,	#(1 << 19)
	str	r1,	[r0, #0x100]

	/* initalize stack pointers and zero out .bss */
	msr	CPSR_c, (0x1f | (1 << 6) | (1 << 7))
	ldr	sp,	=(RAM_START + RAM_SIZE)
	mov	r0,	#0
	ldr	r1, 	=RAM_START
	ldr	r2,	=__bss_end__
2:	cmp	r1,	r2
	strlo	r0,	[r1],	#4
	blo	2b

	b	init
