/* Copyright (C) 2010, Alexander Graf <agraf@znc.in>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#include "path.h"
#include "lib.h"

struct p_mod {
	const struct p_handles *handles;
	const char *name;
	const char *desc;
	struct p_mod *next;
};

static struct p_mod *mods;

void p_regmod(const struct p_handles *handles, const char *name,
	      const char *desc)
{
	struct p_mod *m;
	struct p_mod *newmod = malloc(sizeof(struct p_mod));

	if (!mods)
		mods = newmod;
	else {
		for (m = mods; m->next; m = m->next) ;
		m->next = newmod;
	}

	newmod->handles = handles;
	newmod->name	= name;
	newmod->desc	= desc;
	newmod->next	= 0;
}

void p_listmods()
{
	struct p_mod *m;

	for (m = mods; m; m = m->next)
		printf(" %-5s %s\n", m->name, m->desc);
}

/* setup new instance as child of "parent" */
static struct p_instance *p_setup(struct p_instance *parent,
				  char *name, char *arg)
{
	struct p_mod *m;
	struct p_instance *rv;

	for (m = mods; m; m = m->next)
		if (!strcmp(name, m->name))
			break;
	if (!m) {
		errmsg = "Module not implemented.";
		return 0;
	}

	rv = malloc(sizeof(struct p_instance));
	rv->parent	= parent;
	rv->mod		= m;
	rv->data	= 0;

	if (m->handles->setup)
		if (!m->handles->setup(rv, arg)) {
			free(rv);
			return 0;
		}

	return rv;
}

/* setup instance belonging to a path.
 * This sets up a stack of instances */
struct p_instance *p_open(char *path)
{
	char *modname, *modarg, *nextmodname;
	struct p_instance *inst = 0, *nextinst;

	modname = path;
	do {
		/* nextmodname should point to the next mod name + arg, the
		 * current modname+arg should be \0-terminated, path should
		 * point to the yet-unprocessed part */
		nextmodname = strchr(path, '|');
		if (nextmodname) {
			*nextmodname = 0;
			nextmodname++;
			path = nextmodname;
		}

		/* modarg should point to the argument of the module, modname
		 * should be \0-terminated */
		modarg = strchr(modname, ':');
		if (modarg) {
			*modarg = 0;
			modarg++;
		}

		nextinst = p_setup(inst, modname, modarg);
		if (!nextinst)
			return 0;
		inst = nextinst;
		modname = nextmodname;
	} while (nextmodname);

	if (!inst)
		errmsg = "Path invalid.";

	return inst;
}

int p_read(struct p_instance *inst, void *buf, unsigned off, unsigned int count)
{
	return inst->mod->handles->read(inst, buf, off, count);
}

void p_remove(struct p_instance *inst)
{
	if (inst->mod->handles->remove)
		inst->mod->handles->remove(inst);

	if (inst->parent)
		p_remove(inst->parent);

	free(inst);
}
