/* Copyright (C) 2010, Alexander Graf <agraf@znc.in>
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */

#pragma once

/* defined in path.c */
struct p_mod;

/* instance of a module.  Set up with p_open().  Used to refer to instances in
 * functions such as p_read(). */
struct p_instance {
	/* instance of parent path component */
	struct p_instance *parent;

	/* The module to which this instance belongs */
	struct p_mod *mod;

	/* to be used by module to store data identifying this instance */
	void *data;
};

/* set of functions defined by each module */
struct p_handles {
	/* setup new instance (e.g. open file).  Should return 1 on success and
	 * 0 on fail.
	 * Might be 0. */
	int (*setup)(struct p_instance *, char *arg);

	/* remove instance (module specific data).
	 * Might be 0. */
	void (*remove)(struct p_instance *);

	/* read inside the instance (e.g. return a piece of the file).  Should
	 * return bytes read, 0 on end of file, -1 on error. */
	int (*read)(struct p_instance *, void *buf, unsigned int off,
		    unsigned int count);
};

/* register a module to be used.
 * name is the name of it, desc a brief description displayed on "help". */
extern void p_regmod(const struct p_handles *handles, const char *name,
		     const char *desc);

/* list all modules (called by "help") */
extern void p_listmods();

/* open a file */
extern struct p_instance *p_open(char *path);

/* read in a file */
extern int p_read(struct p_instance *inst, void *buf, unsigned int off,
		  unsigned int count);

/* remove instance */
extern void p_remove(struct p_instance *inst);
